#include <omp.h>     // OpenMP library
#include <stdint.h>  // for uint8_t
#include <stdio.h>   // for printf, (GNU) getline
#include <stdlib.h>  // for malloc, free
#include <string.h>  // for strcmp

void invert(uint8_t* image_data, int width, int height) {
  // TODO Insert the appropriate OpenMP pragma here to 
  // execute the outer loop in parallel in different threads.
  for (int i = 0; i < height; i++) {
    // TODO Insert the appropriate OpenMP pragma here to
    // vectorize the inner loop, i.e., use SIMD instructions.
    for (int j = 0; j < width; j++) {
      image_data[i * width + j] = 255 - image_data[i * width + j];
    }
  }
}

// Do not change this function!
int main(int argc, char** argv) {
  // Open the image file.
  FILE* fp = fopen("stuttgart.pgm", "r");
  if (fp == NULL) {
    printf("Error opening file\n");
    return 1;
  }
  // Read the image header.
  char header[3];
  fscanf(fp, "%2s\n", header);
  if (strcmp(header, "P5") != 0) {
    printf("Error: not a PGM image\n");
    return 1;
  }

  char* comment = NULL;
  size_t comment_size = 0;
  getline(&comment, &comment_size, fp);
  printf("Comment: %s", comment);
  free(comment);

  int width, height;
  fscanf(fp, "%d %d", &width, &height);
  printf("Image dimensions: %d x %d\n", width, height);
  if (width * height > 1000000 || width * height < 0) {
    printf("Error: too large or small image dimensions\n");
    return 1;
  }

  // Read the maximum value.
  int max_value;
  fscanf(fp, "%d\n", &max_value);
  if (max_value != 255) {
    printf("Error: not an 8-bit image\n");
    return 1;
  }

  // Read the raw image data from the file.
  uint8_t* image_data = (uint8_t*)malloc(height * width);
  fread(image_data, sizeof(uint8_t), width * height, fp);

  fclose(fp);

  // Invert the image (and do it often enough to see the effect).
  for (int i = 0; i < 100001; i++) {
    invert(image_data, width, height);
  }

  // Write the image data to a new file.
  FILE* fp_out = fopen("stuttgart_inverted.pgm", "w");
  if (fp_out == NULL) {
    printf("Error creating output file\n");
    return 1;
  }

  fprintf(fp_out, "P5\n");
  fprintf(fp_out, "%d %d\n", width, height);
  fprintf(fp_out, "255\n");
  fwrite(image_data, sizeof(uint8_t), width * height, fp_out);

  fclose(fp_out);

  free(image_data);

  return 0;
}
